@@ -36,12 +36,15 @@ var errNoPackages = errors.New("no packages returned")
 //
 // The resulting error may wrap the moduleErrorMap error type, representing
 // errors associated with specific modules.
+//
+// If scopes contains a file scope there must be exactly one scope.
 func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadScope) (err error) {
 	id := atomic.AddUint64(&loadID, 1)
 	eventName := fmt.Sprintf("go/packages.Load #%d", id) // unique name for logging
 
 	var query []string
 	var containsDir bool // for logging
+	var standalone bool  // whether this is a load of a standalone file
 
 	// Keep track of module query -> module path so that we can later correlate query
 	// errors with errors.
@@ -55,9 +58,16 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			query = append(query, string(scope))
 
 		case fileLoadScope:
+			// Given multiple scopes, the resulting load might contain inaccurate
+			// information. For example go/packages returns at most one command-line
+			// arguments package, and does not handle a combination of standalone
+			// files and packages.
 			uri := span.URI(scope)
+			if len(scopes) > 1 {
+				panic(fmt.Sprintf("internal error: load called with multiple scopes when a file scope is present (file: %s)", uri))
+			}
 			fh := s.FindFile(uri)
-			if fh == nil || s.View().FileKind(fh) != source.Go {
+			if fh == nil || s.FileKind(fh) != source.Go {
 				// Don't try to load a file that doesn't exist, or isn't a go file.
 				continue
 			}
@@ -65,7 +75,8 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			if err != nil {
 				continue
 			}
-			if isStandaloneFile(contents, s.view.Options().StandaloneTags) {
+			if isStandaloneFile(contents, s.options.StandaloneTags) {
+				standalone = true
 				query = append(query, uri.Filename())
 			} else {
 				query = append(query, fmt.Sprintf("file=%s", uri.Filename()))
@@ -79,7 +90,7 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 		case viewLoadScope:
 			// If we are outside of GOPATH, a module, or some other known
 			// build system, don't load subdirectories.
-			if !s.ValidBuildConfiguration() {
+			if !s.validBuildConfiguration() {
 				query = append(query, "./")
 			} else {
 				query = append(query, "./...")
@@ -106,7 +117,7 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 		flags |= source.AllowNetwork
 	}
 	_, inv, cleanup, err := s.goCommandInvocation(ctx, flags, &gocommand.Invocation{
-		WorkingDir: s.view.workingDir().Filename(),
+		WorkingDir: s.view.goCommandDir.Filename(),
 	})
 	if err != nil {
 		return err
@@ -144,33 +155,12 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 		return fmt.Errorf("packages.Load error: %w", err)
 	}
 
-	// Workaround for a bug (?) that has been in go/packages since
-	// the outset: Package("unsafe").GoFiles=[], whereas it should
-	// include unsafe/unsafe.go. Derive it from builtins.go.
-	//
-	// This workaround relies on the fact that we always add both
-	// builtins and unsafe to the set of scopes in the workspace load.
-	//
-	// TODO(adonovan): fix upstream in go/packages.
-	// (Does this need a proposal? Arguably not.)
-	{
-		var builtin, unsafe *packages.Package
-		for _, pkg := range pkgs {
-			if pkg.ID == "unsafe" {
-				unsafe = pkg
-			} else if pkg.ID == "builtin" {
-				builtin = pkg
-			}
-		}
-		if builtin != nil && unsafe != nil && len(builtin.GoFiles) == 1 {
-			unsafe.GoFiles = []string{
-				filepath.Join(filepath.Dir(builtin.GoFiles[0]), "../unsafe/unsafe.go"),
-			}
-		}
+	if standalone && len(pkgs) > 1 {
+		return bug.Errorf("internal error: go/packages returned multiple packages for standalone file")
 	}
 
 	moduleErrs := make(map[string][]packages.Error) // module path -> errors
-	filterFunc := s.view.filterFunc()
+	filterFunc := s.filterFunc()
 	newMetadata := make(map[PackageID]*source.Metadata)
 	for _, pkg := range pkgs {
 		// The Go command returns synthetic list results for module queries that
@@ -188,7 +178,7 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			continue
 		}
 
-		if !containsDir || s.view.Options().VerboseOutput {
+		if !containsDir || s.options.VerboseOutput {
 			event.Log(ctx, eventName, append(
 				source.SnapshotLabels(s),
 				tag.Package.Of(pkg.ID),
@@ -221,14 +211,13 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 		if allFilesExcluded(pkg.GoFiles, filterFunc) {
 			continue
 		}
-		buildMetadata(newMetadata, pkg, cfg.Dir, query)
+		buildMetadata(newMetadata, pkg, cfg.Dir, standalone)
 	}
 
 	s.mu.Lock()
 
 	// Assert the invariant s.packages.Get(id).m == s.meta.metadata[id].
-	s.packages.Range(func(k, v interface{}) {
-		id, ph := k.(PackageID), v.(*packageHandle)
+	s.packages.Range(func(id PackageID, ph *packageHandle) {
 		if s.meta.metadata[id] != ph.m {
 			panic("inconsistent metadata")
 		}
@@ -338,12 +327,12 @@ func (s *snapshot) workspaceLayoutError(ctx context.Context) (error, []*source.D
 
 	// Apply diagnostics about the workspace configuration to relevant open
 	// files.
-	openFiles := s.openFiles()
+	openFiles := s.overlays()
 
 	// If the snapshot does not have a valid build configuration, it may be
 	// that the user has opened a directory that contains multiple modules.
 	// Check for that an warn about it.
-	if !s.ValidBuildConfiguration() {
+	if !s.validBuildConfiguration() {
 		var msg string
 		if s.view.goversion >= 18 {
 			msg = `gopls was not able to find modules in your workspace.
@@ -362,62 +351,15 @@ https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.`
 		return fmt.Errorf(msg), s.applyCriticalErrorToFiles(ctx, msg, openFiles)
 	}
 
-	// If the user has one active go.mod file, they may still be editing files
-	// in nested modules. Check the module of each open file and add warnings
-	// that the nested module must be opened as a workspace folder.
-	if len(s.workspaceModFiles) == 1 {
-		// Get the active root go.mod file to compare against.
-		var rootMod string
-		for uri := range s.workspaceModFiles {
-			rootMod = uri.Filename()
-		}
-		rootDir := filepath.Dir(rootMod)
-		nestedModules := make(map[string][]source.FileHandle)
-		for _, fh := range openFiles {
-			mod, err := findRootPattern(ctx, filepath.Dir(fh.URI().Filename()), "go.mod", s)
-			if err != nil {
-				if ctx.Err() != nil {
-					return ctx.Err(), nil
-				}
-				continue
-			}
-			if mod == "" {
-				continue
-			}
-			if mod != rootMod && source.InDir(rootDir, mod) {
-				modDir := filepath.Dir(mod)
-				nestedModules[modDir] = append(nestedModules[modDir], fh)
-			}
-		}
-		var multiModuleMsg string
-		if s.view.goversion >= 18 {
-			multiModuleMsg = `To work on multiple modules at once, please use a go.work file.
-See https://github.com/golang/tools/blob/master/gopls/doc/workspace.md for more information on using workspaces.`
-		} else {
-			multiModuleMsg = `To work on multiple modules at once, please upgrade to Go 1.18 and use a go.work file.
-See https://github.com/golang/tools/blob/master/gopls/doc/workspace.md for more information on using workspaces.`
-		}
-		// Add a diagnostic to each file in a nested module to mark it as
-		// "orphaned". Don't show a general diagnostic in the progress bar,
-		// because the user may still want to edit a file in a nested module.
-		var srcDiags []*source.Diagnostic
-		for modDir, uris := range nestedModules {
-			msg := fmt.Sprintf("This file is in %s, which is a nested module in the %s module.\n%s", modDir, rootMod, multiModuleMsg)
-			srcDiags = append(srcDiags, s.applyCriticalErrorToFiles(ctx, msg, uris)...)
-		}
-		if len(srcDiags) != 0 {
-			return fmt.Errorf("You have opened a nested module.\n%s", multiModuleMsg), srcDiags
-		}
-	}
 	return nil, nil
 }
 
-func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []source.FileHandle) []*source.Diagnostic {
+func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []*Overlay) []*source.Diagnostic {
 	var srcDiags []*source.Diagnostic
 	for _, fh := range files {
 		// Place the diagnostics on the package or module declarations.
 		var rng protocol.Range
-		switch s.view.FileKind(fh) {
+		switch s.FileKind(fh) {
 		case source.Go:
 			if pgf, err := s.ParseGo(ctx, fh, source.ParseHeader); err == nil {
 				// Check that we have a valid `package foo` range to use for positioning the error.
@@ -446,19 +388,17 @@ func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, fi
 // buildMetadata populates the updates map with metadata updates to
 // apply, based on the given pkg. It recurs through pkg.Imports to ensure that
 // metadata exists for all dependencies.
-func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package, loadDir string, query []string) {
+func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package, loadDir string, standalone bool) {
 	// Allow for multiple ad-hoc packages in the workspace (see #47584).
 	pkgPath := PackagePath(pkg.PkgPath)
 	id := PackageID(pkg.ID)
 
-	// TODO(rfindley): this creates at most one command-line-arguments package
-	// per load, but if we pass multiple file= queries to go/packages, there may
-	// be multiple command-line-arguments packages.
-	//
-	// As reported in golang/go#59318, this can result in accidentally quadratic
-	// loading behavior.
 	if source.IsCommandLineArguments(id) {
-		suffix := ":" + strings.Join(query, ",")
+		if len(pkg.CompiledGoFiles) != 1 {
+			bug.Reportf("unexpected files in command-line-arguments package: %v", pkg.CompiledGoFiles)
+			return
+		}
+		suffix := pkg.CompiledGoFiles[0]
 		id = PackageID(pkg.ID + suffix)
 		pkgPath = PackagePath(pkg.PkgPath + suffix)
 	}
@@ -484,6 +424,7 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 		Module:     pkg.Module,
 		Errors:     pkg.Errors,
 		DepsErrors: packagesinternal.GetDepsErrors(pkg),
+		Standalone: standalone,
 	}
 
 	updates[id] = m
@@ -496,6 +437,10 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 		uri := span.URIFromPath(filename)
 		m.GoFiles = append(m.GoFiles, uri)
 	}
+	for _, filename := range pkg.IgnoredFiles {
+		uri := span.URIFromPath(filename)
+		m.IgnoredFiles = append(m.IgnoredFiles, uri)
+	}
 
 	depsByImpPath := make(map[ImportPath]PackageID)
 	depsByPkgPath := make(map[PackagePath]PackageID)
@@ -582,9 +527,21 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 			continue
 		}
 
+		buildMetadata(updates, imported, loadDir, false) // only top level packages can be standalone
+
+		// Don't record edges to packages with no name, as they cause trouble for
+		// the importer (golang/go#60952).
+		//
+		// However, we do want to insert these packages into the update map
+		// (buildMetadata above), so that we get type-checking diagnostics for the
+		// invalid packages.
+		if imported.Name == "" {
+			depsByImpPath[importPath] = "" // missing
+			continue
+		}
+
 		depsByImpPath[importPath] = PackageID(imported.ID)
 		depsByPkgPath[PackagePath(imported.PkgPath)] = PackageID(imported.ID)
-		buildMetadata(updates, imported, loadDir, query)
 	}
 	m.DepsByImpPath = depsByImpPath
 	m.DepsByPkgPath = depsByPkgPath
@@ -659,7 +616,7 @@ func containsPackageLocked(s *snapshot, m *source.Metadata) bool {
 			uris[uri] = struct{}{}
 		}
 
-		filterFunc := s.view.filterFunc()
+		filterFunc := s.filterFunc()
 		for uri := range uris {
 			// Don't use view.contains here. go.work files may include modules
 			// outside of the workspace folder.
@@ -687,7 +644,8 @@ func containsOpenFileLocked(s *snapshot, m *source.Metadata) bool {
 	}
 
 	for uri := range uris {
-		if s.isOpenLocked(uri) {
+		fh, _ := s.files.Get(uri)
+		if _, open := fh.(*Overlay); open {
 			return true
 		}
 	}
@@ -713,7 +671,7 @@ func containsFileInWorkspaceLocked(s *snapshot, m *source.Metadata) bool {
 
 		// The package's files are in this view. It may be a workspace package.
 		// Vendored packages are not likely to be interesting to the user.
-		if !strings.Contains(string(uri), "/vendor/") && s.view.contains(uri) {
+		if !strings.Contains(string(uri), "/vendor/") && s.contains(uri) {
 			return true
 		}
 	}
