@@ -52,7 +52,7 @@ func (s *snapshot) ParseMod(ctx context.Context, fh source.FileHandle) (*source.
 	}
 
 	// Await result.
-	v, err := s.awaitPromise(ctx, entry.(*memoize.Promise))
+	v, err := s.awaitPromise(ctx, entry)
 	if err != nil {
 		return nil, err
 	}
@@ -130,7 +130,7 @@ func (s *snapshot) ParseWork(ctx context.Context, fh source.FileHandle) (*source
 	}
 
 	// Await result.
-	v, err := s.awaitPromise(ctx, entry.(*memoize.Promise))
+	v, err := s.awaitPromise(ctx, entry)
 	if err != nil {
 		return nil, err
 	}
@@ -213,7 +213,7 @@ func sumFilename(modURI span.URI) string {
 func (s *snapshot) ModWhy(ctx context.Context, fh source.FileHandle) (map[string]string, error) {
 	uri := fh.URI()
 
-	if s.View().FileKind(fh) != source.Mod {
+	if s.FileKind(fh) != source.Mod {
 		return nil, fmt.Errorf("%s is not a go.mod file", uri)
 	}
 
@@ -240,7 +240,7 @@ func (s *snapshot) ModWhy(ctx context.Context, fh source.FileHandle) (map[string
 	}
 
 	// Await result.
-	v, err := s.awaitPromise(ctx, entry.(*memoize.Promise))
+	v, err := s.awaitPromise(ctx, entry)
 	if err != nil {
 		return nil, err
 	}
@@ -400,7 +400,7 @@ func (s *snapshot) matchErrorToModule(ctx context.Context, pm *source.ParsedModu
 		// Show the error on the module declaration, if one exists, or
 		// just the first line of the file.
 		var start, end int
-		if pm.File.Module != nil {
+		if pm.File.Module != nil && pm.File.Module.Syntax != nil {
 			syntax := pm.File.Module.Syntax
 			start, end = syntax.Start.Byte, syntax.End.Byte
 		}
