@@ -23,6 +23,7 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/template"
 	"golang.org/x/tools/internal/event"
+	"golang.org/x/tools/internal/event/tag"
 	"golang.org/x/tools/internal/typeparams"
 )
 
@@ -37,25 +38,22 @@ const maxFullFileSize int = 100000
 // semDebug should NEVER be true in checked-in code
 const semDebug = false
 
-func (s *Server) semanticTokensFull(ctx context.Context, p *protocol.SemanticTokensParams) (*protocol.SemanticTokens, error) {
-	ret, err := s.computeSemanticTokens(ctx, p.TextDocument, nil)
+func (s *Server) semanticTokensFull(ctx context.Context, params *protocol.SemanticTokensParams) (*protocol.SemanticTokens, error) {
+	ctx, done := event.Start(ctx, "lsp.Server.semanticTokensFull", tag.URI.Of(params.TextDocument.URI))
+	defer done()
+
+	ret, err := s.computeSemanticTokens(ctx, params.TextDocument, nil)
 	return ret, err
 }
 
-func (s *Server) semanticTokensFullDelta(ctx context.Context, p *protocol.SemanticTokensDeltaParams) (interface{}, error) {
-	return nil, fmt.Errorf("implement SemanticTokensFullDelta")
-}
+func (s *Server) semanticTokensRange(ctx context.Context, params *protocol.SemanticTokensRangeParams) (*protocol.SemanticTokens, error) {
+	ctx, done := event.Start(ctx, "lsp.Server.semanticTokensRange", tag.URI.Of(params.TextDocument.URI))
+	defer done()
 
-func (s *Server) semanticTokensRange(ctx context.Context, p *protocol.SemanticTokensRangeParams) (*protocol.SemanticTokens, error) {
-	ret, err := s.computeSemanticTokens(ctx, p.TextDocument, &p.Range)
+	ret, err := s.computeSemanticTokens(ctx, params.TextDocument, &params.Range)
 	return ret, err
 }
 
-func (s *Server) semanticTokensRefresh(ctx context.Context) error {
-	// in the code, but not in the protocol spec
-	return fmt.Errorf("implement SemanticTokensRefresh")
-}
-
 func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocumentIdentifier, rng *protocol.Range) (*protocol.SemanticTokens, error) {
 	ans := protocol.SemanticTokens{
 		Data: []uint32{},
@@ -65,13 +63,12 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 	if !ok {
 		return nil, err
 	}
-	vv := snapshot.View()
-	if !vv.Options().SemanticTokens {
+	if !snapshot.Options().SemanticTokens {
 		// return an error, so if the option changes
 		// the client won't remember the wrong answer
 		return nil, fmt.Errorf("semantictokens are disabled")
 	}
-	kind := snapshot.View().FileKind(fh)
+	kind := snapshot.FileKind(fh)
 	if kind == source.Tmpl {
 		// this is a little cumbersome to avoid both exporting 'encoded' and its methods
 		// and to avoid import cycles
@@ -79,8 +76,8 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 			ctx:            ctx,
 			metadataSource: snapshot,
 			rng:            rng,
-			tokTypes:       s.session.Options().SemanticTypes,
-			tokMods:        s.session.Options().SemanticMods,
+			tokTypes:       snapshot.Options().SemanticTypes,
+			tokMods:        snapshot.Options().SemanticMods,
 		}
 		add := func(line, start uint32, len uint32) {
 			e.add(line, start, len, tokMacro, nil)
@@ -93,7 +90,7 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 	if kind != source.Go {
 		return nil, nil
 	}
-	pkg, pgf, err := source.PackageForFile(ctx, snapshot, fh.URI(), source.NarrowestPackage)
+	pkg, pgf, err := source.NarrowestPackageForFile(ctx, snapshot, fh.URI())
 	if err != nil {
 		return nil, err
 	}
@@ -111,10 +108,10 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 		ti:             pkg.GetTypesInfo(),
 		pkg:            pkg,
 		fset:           pkg.FileSet(),
-		tokTypes:       s.session.Options().SemanticTypes,
-		tokMods:        s.session.Options().SemanticMods,
-		noStrings:      vv.Options().NoSemanticString,
-		noNumbers:      vv.Options().NoSemanticNumber,
+		tokTypes:       snapshot.Options().SemanticTypes,
+		tokMods:        snapshot.Options().SemanticMods,
+		noStrings:      snapshot.Options().NoSemanticString,
+		noNumbers:      snapshot.Options().NoSemanticNumber,
 	}
 	if err := e.init(); err != nil {
 		// e.init should never return an error, unless there's some
@@ -258,7 +255,7 @@ func (e *encoded) strStack() string {
 // find the line in the source
 func (e *encoded) srcLine(x ast.Node) string {
 	file := e.pgf.Tok
-	line := file.Line(x.Pos())
+	line := safetoken.Line(file, x.Pos())
 	start, err := safetoken.Offset(file, file.LineStart(line))
 	if err != nil {
 		return ""
@@ -839,17 +836,16 @@ func (e *encoded) findKeyword(keyword string, start, end token.Pos) token.Pos {
 }
 
 func (e *encoded) init() error {
-	e.start = token.Pos(e.pgf.Tok.Base())
-	e.end = e.start + token.Pos(e.pgf.Tok.Size())
-	if e.rng == nil {
-		return nil
+	if e.rng != nil {
+		var err error
+		e.start, e.end, err = e.pgf.RangePos(*e.rng)
+		if err != nil {
+			return fmt.Errorf("range span (%w) error for %s", err, e.pgf.File.Name)
+		}
+	} else {
+		tok := e.pgf.Tok
+		e.start, e.end = tok.Pos(0), tok.Pos(tok.Size()) // entire file
 	}
-	span, err := e.pgf.Mapper.RangeSpan(*e.rng)
-	if err != nil {
-		return fmt.Errorf("range span (%w) error for %s", err, e.pgf.File.Name)
-	}
-	e.end = e.start + token.Pos(span.End().Offset())
-	e.start += token.Pos(span.Start().Offset())
 	return nil
 }
 
