@@ -9,7 +9,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -45,7 +44,7 @@ func GCOptimizationDetails(ctx context.Context, snapshot Snapshot, m *Metadata)
 	if err := os.MkdirAll(outDir, 0700); err != nil {
 		return nil, err
 	}
-	tmpFile, err := ioutil.TempFile(os.TempDir(), "gopls-x")
+	tmpFile, err := os.CreateTemp(os.TempDir(), "gopls-x")
 	if err != nil {
 		return nil, err
 	}
@@ -75,7 +74,7 @@ func GCOptimizationDetails(ctx context.Context, snapshot Snapshot, m *Metadata)
 		return nil, err
 	}
 	reports := make(map[span.URI][]*Diagnostic)
-	opts := snapshot.View().Options()
+	opts := snapshot.Options()
 	var parseError error
 	for _, fn := range files {
 		uri, diagnostics, err := parseDetailsFile(fn, opts)
@@ -99,7 +98,7 @@ func GCOptimizationDetails(ctx context.Context, snapshot Snapshot, m *Metadata)
 }
 
 func parseDetailsFile(filename string, options *Options) (span.URI, []*Diagnostic, error) {
-	buf, err := ioutil.ReadFile(filename)
+	buf, err := os.ReadFile(filename)
 	if err != nil {
 		return "", nil, err
 	}
@@ -129,6 +128,7 @@ func parseDetailsFile(filename string, options *Options) (span.URI, []*Diagnosti
 		if err := dec.Decode(d); err != nil {
 			return "", nil, err
 		}
+		d.Tags = []protocol.DiagnosticTag{} // must be an actual slice
 		msg := d.Code.(string)
 		if msg != "" {
 			msg = fmt.Sprintf("%s(%s)", msg, d.Message)
@@ -136,11 +136,18 @@ func parseDetailsFile(filename string, options *Options) (span.URI, []*Diagnosti
 		if !showDiagnostic(msg, d.Source, options) {
 			continue
 		}
-		var related []RelatedInformation
+		var related []protocol.DiagnosticRelatedInformation
 		for _, ri := range d.RelatedInformation {
-			related = append(related, RelatedInformation{
-				URI:     ri.Location.URI.SpanURI(),
-				Range:   zeroIndexedRange(ri.Location.Range),
+			// TODO(rfindley): The compiler uses LSP-like JSON to encode gc details,
+			// however the positions it uses are 1-based UTF-8:
+			// https://github.com/golang/go/blob/master/src/cmd/compile/internal/logopt/log_opts.go
+			//
+			// Here, we adjust for 0-based positions, but do not translate UTF-8 to UTF-16.
+			related = append(related, protocol.DiagnosticRelatedInformation{
+				Location: protocol.Location{
+					URI:   ri.Location.URI,
+					Range: zeroIndexedRange(ri.Location.Range),
+				},
 				Message: ri.Message,
 			})
 		}
