@@ -42,10 +42,8 @@ var FooErr = errors.New("foo")
 		cfg.Settings = map[string]interface{}{
 			"staticcheck": true,
 		}
-		// TODO(rfindley): support waiting on diagnostics following a configuration
-		// change.
 		env.ChangeConfiguration(cfg)
-		env.Await(
+		env.AfterChange(
 			Diagnostics(env.AtRegexp("a/a.go", "var (FooErr)")),
 		)
 	})
@@ -89,10 +87,8 @@ var ErrFoo = errors.New("foo")
 		cfg.Settings = map[string]interface{}{
 			"staticcheck": true,
 		}
-		// TODO(rfindley): support waiting on diagnostics following a configuration
-		// change.
 		env.ChangeConfiguration(cfg)
-		env.Await(
+		env.AfterChange(
 			Diagnostics(env.AtRegexp("a/a.go", "var (FooErr)")),
 		)
 	})
